// Copyright (C) WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:architecture_example/data/data_sources/remote/album_remote_data_source.dart';
import 'package:architecture_example/data/models/album_model.dart';
import 'package:architecture_example/data/models/list_album_model.dart';
import 'package:architecture_example/domain/entities/album.dart';
import 'package:wyatt_http_client/wyatt_http_client.dart';

class AlbumApiDataSourceImpl implements AlbumRemoteDataSource {
  AlbumApiDataSourceImpl(this._client);
  final MiddlewareClient _client;

  @override
  Future<Album> getAlbum(int id) async {
    final response = await _client.get(Uri.parse('/albums/$id'));
    final album =
        AlbumModel.fromJson(jsonDecode(response.body) as Map<String, Object?>);
    return album;
  }

  @override
  Future<List<Album>> getAllAlbums({int? start, int? limit}) async {
    final startQuery = (start != null) ? '_start=$start' : '';
    final limitQuery = (limit != null) ? '_limit=$limit' : '';
    final delimiter1 =
        (startQuery.isNotEmpty || limitQuery.isNotEmpty) ? '?' : '';
    final delimiter2 =
        (startQuery.isNotEmpty && limitQuery.isNotEmpty) ? '&' : '';
    final url = '/albums$delimiter1$startQuery$delimiter2$limitQuery';
    final response = await _client.get(Uri.parse(url));
    final albums =
        ListAlbumModel.fromJson({'albums': jsonDecode(response.body)});
    return albums.albums;
  }
}
