// Copyright (C) WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

/// {@template app_exception}
/// [AppException] is a base class for all exceptions in the wyatt architecture.
/// {@endtemplate}
abstract class AppException implements Exception {
  /// {@macro app_exception}
  const AppException({this.message, this.underlyingException});

  /// {@macro app_exception}
  /// 
  /// Constructs an [AppException] from an [Exception].
  factory AppException.from(Object e) {
    if (e is AppException) {
      return e;
    } else {
      return ClientException(underlyingException: e);
    }
  }

  /// The message of the exception.
  final String? message;

  /// The exception which triggered this exception being thrown.
  final Object? underlyingException;

  @override
  String toString() {
    final str = StringBuffer(runtimeType.toString());
    if (message != null && message!.isNotEmpty) {
      str.write(': $message');
    } else {
      str.write(': An exception occured');
    }
    if (underlyingException != null) {
      str.write('\n$underlyingException');
    }

    return str.toString();
  }
}

/// {@template client_exception}
/// [ClientException] is a base class for all client exceptions in the wyatt
/// architecture.
/// {@endtemplate}
class ClientException extends AppException {
  /// {@macro client_exception}
  const ClientException({super.message, super.underlyingException});
}

/// {@template server_exception}
/// [ServerException] is a base class for all server exceptions in the wyatt
/// architecture.
/// {@endtemplate}
class ServerException extends AppException {
  /// {@macro server_exception}
  const ServerException({super.message, super.underlyingException});
}
